--[[

	UI element creation/tracking for EnergyWatch

--]]


--[[ Globals --]]
EnergyWatchUI = {}

--[[ Main addon frame creation and main local vars ]]--

function EnergyWatchUI.CreateEnergyBar()
	local barWidth = EnergyWatch.GetConfigValue("barWidth");
	local energyBar = CreateFrame("Frame", "EnergyWatchBar", UIParent)
	energyBar:SetSize(barWidth, 26)
	energyBar:SetMovable(true)

	energyBar:SetScript("OnMouseDown", function (self, button)
		if button == "LeftButton" then
 			self:StartMoving();
 		end
	end)
	energyBar:SetScript("OnMouseUp", function (self, button)
		if button == "LeftButton" then
 			self:StopMovingOrSizing();
			local point, relativeTo, relativePoint, xOffset, yOffset = self:GetPoint(1)

			EnergyWatch.SetConfigValue("barPosX", xOffset)
			EnergyWatch.SetConfigValue("barPosY", yOffset)
			EnergyWatch.SetConfigValue("barPosPoint", point)
 		end
	end)

	if EnergyWatch.GetConfigValue("locked") then
		energyBar:EnableMouse(false)
	else 
		energyBar:EnableMouse(true)
	end

	energyBar:SetPoint(EnergyWatch.GetConfigValue("barPosPoint"), EnergyWatch.GetConfigValue("barPosX"), EnergyWatch.GetConfigValue("barPosY"))
	energyBar:Hide()

	local energyBarBackground = energyBar:CreateTexture("EnergyWatchBarBackground", "BACKGROUND")
	energyBarBackground:SetSize(barWidth - 9, 13)
	energyBarBackground:SetTexture(0, 0, 0, .5)
	energyBarBackground:SetPoint("TOP", 0, -2)

	local energyBarBorder = energyBar:CreateTexture("EnergyWatchBarBorder", "OVERLAY")
	energyBarBorder:SetSize(barWidth - 5, 20)
	energyBarBorder:SetTexture("Interface\\Tooltips\\UI-StatusBar-Border")
	energyBarBorder:SetPoint("TOP", 0, 2)

	local energyBarText = energyBar:CreateFontString("EnergyWatchText", "OVERLAY", "GameFontHighlight")
	energyBarText:SetSize(barWidth - 6, 16)
	energyBarText:SetPoint("TOP", 0, 0)

	local energyBarStatus = CreateFrame("StatusBar", "EnergyWatchStatusBar", energyBar)
	energyBarStatus:SetSize(barWidth - 11, 13)
	energyBarStatus:SetPoint("TOP", 0, -2)
	energyBarStatus:SetFrameLevel(energyBarStatus:GetParent():GetFrameLevel())

	local energyBarStatusTexture = energyBarStatus:CreateTexture("EnergyWatchStatusBarTexture", "BACKGROUND")
	energyBarStatusTexture:SetTexture("Interface\\TargetingFrame\\UI-StatusBar")

	energyBarStatus:SetStatusBarTexture(energyBarStatusTexture)

end

--[[ Config UI --]]
function EnergyWatchUI.CreateConfigMenu()
	local configPanel = CreateFrame("Frame", "EnergyWatchConfigFrame", UIParent)
	configPanel.name = "EnergyWatch"
	configPanel.okay = function (self) return end
	configPanel.cancel = function (self) return end

	local addonName, addonTitle, addonNotes = GetAddOnInfo('EnergyWatch')
	local configPanelText = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	configPanelText:SetPoint('TOPLEFT', 16, -16)
	configPanelText:SetText(addonTitle)

	local configPanelDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	configPanelDesc:SetPoint('TOPLEFT', configPanelText, 'BOTTOMLEFT', 0, -8)
	configPanelDesc:SetPoint('RIGHT', configPanel, -32, 0)
	configPanelDesc:SetWidth(600)
	configPanelDesc:SetJustifyH('LEFT')
	configPanelDesc:SetJustifyV('TOP')
	configPanelDesc:SetText(addonNotes)

	local appearanceConfigPanel = CreateFrame("Frame", "EnergyWatchAppearanceConfigFrame", configPanel)
	appearanceConfigPanel.name = "Appearance Options"
	appearanceConfigPanel.parent = "EnergyWatch"
	appearanceConfigPanel.okay = function (self) return end
	appearanceConfigPanel.cancel = function (self) return end

	local appearanceConfigPanelTitle = appearanceConfigPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	appearanceConfigPanelTitle:SetPoint('TOPLEFT', 16, -16)
	appearanceConfigPanelTitle:SetText(appearanceConfigPanel.name)

	local appearanceConfigPanelDesc = appearanceConfigPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	appearanceConfigPanelDesc:SetHeight(32)
	appearanceConfigPanelDesc:SetPoint('TOPLEFT', appearanceConfigPanelTitle, 'BOTTOMLEFT', 0, -8)
	appearanceConfigPanelDesc:SetPoint('RIGHT', appearanceConfigPanel, -32, 0)
	appearanceConfigPanelDesc:SetNonSpaceWrap(true)
	appearanceConfigPanelDesc:SetJustifyH('LEFT')
	appearanceConfigPanelDesc:SetJustifyV('TOP')
	appearanceConfigPanelDesc:SetText("These options allow you to control the appearance of EnergyWatch's bar")

	local powerTypeOptionDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	powerTypeOptionDesc:SetPoint('TOPLEFT', configPanelDesc, 'BOTTOMLEFT', -2, -10)
	powerTypeOptionDesc:SetText("Which power types to use Energy Watch for")

	local powerTypeEnergy = EnergyWatchUI.CreateCheckButton("Energy", configPanel,"Use Energy Watch when you have Energy", "powerTypeEnergy", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	powerTypeEnergy:SetPoint('TOPLEFT', powerTypeOptionDesc, 'BOTTOMLEFT', 0, 0)

	local powerTypeFocus = EnergyWatchUI.CreateCheckButton("Focus", configPanel,"Use Energy Watch when you have Focus", "powerTypeFocus", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	powerTypeFocus:SetPoint('TOPLEFT', powerTypeEnergy, 'TOPRIGHT', 80, 0)

	local powerTypeMana = EnergyWatchUI.CreateCheckButton("Mana", configPanel,"Use Energy Watch when you have Mana", "powerTypeMana", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	powerTypeMana:SetPoint('TOPLEFT', powerTypeFocus, 'TOPRIGHT', 80, 0)

	local powerTypeRage = EnergyWatchUI.CreateCheckButton("Rage", configPanel,"Use Energy Watch when you have Rage", "powerTypeRage", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	powerTypeRage:SetPoint('TOPLEFT', powerTypeEnergy, 'BOTTOMLEFT', 0, 0)

	local powerTypeRunicPower = EnergyWatchUI.CreateCheckButton("Runic Power", configPanel,"Use Energy Watch when you have Runic Power", "powerTypeRunicPower", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	powerTypeRunicPower:SetPoint('TOPLEFT', powerTypeFocus, 'BOTTOMLEFT', 0, 0)

	local showOptionDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	showOptionDesc:SetPoint('TOPLEFT', powerTypeRage, 'BOTTOMLEFT', -2, -10)
	showOptionDesc:SetText("When to show Energy Watch")

	local showAlways = EnergyWatchUI.CreateCheckButton("Always", configPanel,"Always show Energy Watch\n(Bar will only show if your character is currently using a power type selected above)", "showAlways", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	showAlways:SetPoint('TOPLEFT', showOptionDesc, 'BOTTOMLEFT', 0, 0)

	local showStealth = EnergyWatchUI.CreateCheckButton("In Stealth", configPanel,"Show Energy Watch while stealthed", "showStealth", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	showStealth:SetPoint('TOPLEFT', showAlways, 'BOTTOMLEFT', 10, 0)
	EnergyWatchUI.SetupInverseDependentControl(showAlways, showStealth)

	local showCombat = EnergyWatchUI.CreateCheckButton("In Combat", configPanel,"Show Energy Watch while in combat", "showCombat", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	showCombat:SetPoint('TOPLEFT', showStealth, 'BOTTOMLEFT', 0, 0)
	EnergyWatchUI.SetupInverseDependentControl(showAlways, showCombat)

	local showNonDefault = EnergyWatchUI.CreateCheckButton("When Not Default", configPanel,"Show Energy Watch when your power type is not at default\n(0 for Rage/Runic Power, Max for others)", "showNonDefault", EnergyWatchUI.BooleanCheckButtonClicked, 'InterfaceOptionsCheckButtonTemplate')
	showNonDefault:SetPoint('TOPLEFT', showStealth, 'TOPRIGHT', 80, 0)
	EnergyWatchUI.SetupInverseDependentControl(showAlways, showNonDefault)

	local textEditDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	textEditDesc:SetPoint('TOPLEFT', showCombat, 'BOTTOMLEFT', -10, -10)
	textEditDesc:SetText("Bar Text")

	local textEditBox = EnergyWatchUI.CreateEditBox("Normal", configPanel, "Text to show normally", "barText", EnergyWatchUI.TextEditBoxChanged, "InputBoxTemplate")
	textEditBox:SetPoint('TOPLEFT', textEditDesc, 'BOTTOMLEFT', 80, 0)

	local pointsTextEditBox = EnergyWatchUI.CreateEditBox("With Points", configPanel, "Text to show when you are playing a class/form that has Combo Points/Soul Shards/Holy Power", "barPointsText", EnergyWatchUI.TextEditBoxChanged, "InputBoxTemplate")
	pointsTextEditBox:SetPoint('TOPLEFT', textEditBox, 'BOTTOMLEFT', 0, 0)

	local textEditHint = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	textEditHint:SetPoint('TOPLEFT', pointsTextEditBox, 'BOTTOMLEFT', -80, 0)
	textEditHint:SetText("&e = Energy, &em = Max Energy, &ep = Energy Percentage &c = Special Points")

	local barAlphaDesc = appearanceConfigPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	barAlphaDesc:SetPoint('TOPLEFT', appearanceConfigPanelDesc, 'BOTTOMLEFT', -2, -10)
	barAlphaDesc:SetText("Bar Alpha (Opacity): ")

	local barAlphaSlider = EnergyWatchUI.CreateSlider("BarAlphaSlider", appearanceConfigPanel, "barAlpha", EnergyWatchUI.BarAlphaSliderChanged, 0.0, 1.0, "0.0", "1.0", 0.01, "OptionsSliderTemplate")
	barAlphaSlider:SetPoint('TOPLEFT', barAlphaDesc, 'BOTTOMLEFT', 10, 0)

	local barScaleDesc = appearanceConfigPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	barScaleDesc:SetPoint('TOPLEFT', barAlphaSlider, 'BOTTOMLEFT', -10, -10)
	barScaleDesc:SetText("Bar Scale: ")

	local barScaleSlider = EnergyWatchUI.CreateSlider("BarScaleSlider", appearanceConfigPanel, "barScale", EnergyWatchUI.BarScaleSliderChanged, 0.25, 3.0, "0.25", "3.0", 0.01, "OptionsSliderTemplate")
	barScaleSlider:SetPoint('TOPLEFT', barScaleDesc, 'BOTTOMLEFT', 10, 0)

	local barWidthDesc = appearanceConfigPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	barWidthDesc:SetPoint('TOPLEFT', barScaleSlider, 'BOTTOMLEFT', -10, -10)
	barWidthDesc:SetText("Bar Width: ")

	local barWidthSlider = EnergyWatchUI.CreateSlider("barWidthSlider", appearanceConfigPanel, "barWidth", EnergyWatchUI.BarWidthSliderChanged, 100, 300, "100", "300", 1, "OptionsSliderTemplate")
	barWidthSlider:SetPoint('TOPLEFT', barWidthDesc, 'BOTTOMLEFT', 10, 0)

	InterfaceOptions_AddCategory(configPanel)
	InterfaceOptions_AddCategory(appearanceConfigPanel)
end

function EnergyWatchUI.TextEditBoxChanged(self, isUserInput)
	if isUserInput then
		--print("Caught change, changing " .. self.configKey)
		EnergyWatch.SetConfigValue(self.configKey, self:GetText())
		EnergyWatch.UpdateBar()
	end
end

function EnergyWatchUI.BooleanCheckButtonClicked(self)
	if self:GetChecked() then
		EnergyWatch.SetConfigValue(self.configKey, true)
	else 
		EnergyWatch.SetConfigValue(self.configKey, false)
	end
	if ( self.dependentControls ) then
		if ( self:GetChecked() ) then
			for _, control in pairs(self.dependentControls) do
				control:Enable()
			end
		else
			for _, control in pairs(self.dependentControls) do
				control:Disable()
			end
		end
	end
	if ( self.inverseDependentControls ) then
		if ( self:GetChecked() ) then
			for _, control in pairs(self.inverseDependentControls) do
				control:Disable()
			end
		else
			for _, control in pairs(self.inverseDependentControls) do
				control:Enable()
			end
		end
	end
	EnergyWatch.ShowOrHideBar()
end

function EnergyWatchUI.BarAlphaSliderChanged(self, value)
	EnergyWatchUI.SliderChanged(self, value)

	EnergyWatchBar:SetAlpha(value)
end

function EnergyWatchUI.BarScaleSliderChanged(self, value)
	EnergyWatchUI.SliderChanged(self, value)

	EnergyWatchBar:SetScale(value)
end

function EnergyWatchUI.BarWidthSliderChanged(self, value)
	EnergyWatchUI.SliderChanged(self, value)

	local barWidth = value;
	EnergyWatchBar:SetWidth(barWidth)
	EnergyWatchBarBackground:SetWidth(barWidth - 9)
	EnergyWatchBarBorder:SetWidth(barWidth - 5)
	EnergyWatchText:SetWidth(barWidth - 6)
	EnergyWatchStatusBar:SetWidth(barWidth - 11)
end

function EnergyWatchUI.SliderChanged(self, value)
	EnergyWatch.SetConfigValue(self.configKey, value)

	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(string.format("%.2f", value), nil, nil, nil, 1, 1)
	GameTooltip:Show()

end

function EnergyWatchUI.CreateCheckButton(name, parent, tooltipText, configKey, onClickFunc, template)
	local button = CreateFrame('CheckButton', parent:GetName() .. name, parent, template)
	button.configKey = configKey
	_G[button:GetName() .. 'Text']:SetText(name)
	button:SetChecked(EnergyWatch.GetConfigValue(configKey))
	button:SetScript('OnClick', onClickFunc)

	button:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(tooltipText, nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	button:SetScript('OnLeave', function(self) GameTooltip:Hide() end)

	return button
end

function EnergyWatchUI.CreateEditBox(name, parent, tooltipText, configKey, onTextChangeFunc, template)
	local editBox = CreateFrame("EditBox",  parent:GetName() .. name, parent, template)
	editBox.configKey = configKey
	editBox:SetAutoFocus(false)
	editBox:SetSize(200, 32)
	editBox:SetMaxLetters(128)
	editBox:SetText(EnergyWatch.GetConfigValue(configKey))
	editBox:SetCursorPosition(0)
	editBox:SetScript("OnTextChanged", onTextChangeFunc)

	local editBoxLabel = editBox:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	editBoxLabel:SetText(name .. ":")
	editBoxLabel:SetPoint('RIGHT', editBox, 'LEFT', -5, 0)

	editBox:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(tooltipText, nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	editBox:SetScript('OnLeave', function(self) GameTooltip:Hide() end)

	return editBox
end

function EnergyWatchUI.CreateSlider(name, parent, configKey, onValueChangedFunc, minVal, maxVal, lowText, highText, valStep, template)
	local slider = CreateFrame("Slider", parent:GetName() .. name, parent, template)
	slider.configKey = configKey
	slider:SetMinMaxValues(minVal, maxVal)
	slider:SetValueStep(valStep)
	slider:SetValue(EnergyWatch.GetConfigValue(configKey))
	_G[slider:GetName() .. "Low"]:SetText(lowText)
	_G[slider:GetName() .. "High"]:SetText(highText)
	slider:SetScript("OnValueChanged", onValueChangedFunc)

	slider:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(string.format("%.2f", slider:GetValue()), nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	slider:SetScript('OnLeave', function(self) GameTooltip:Hide() end)


	return slider
end

function EnergyWatchUI.SetupDependentControl (dependency, control)
	if ( not dependency ) then
		return
	end
	
	assert(control)
	
	dependency.dependentControls = dependency.dependentControls or {}
	tinsert(dependency.dependentControls, control)

	if ( control.type ~= CONTROLTYPE_DROPDOWN ) then
		control.Disable = function (self) getmetatable(self).__index.Disable(self) _G[self:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b) end
		control.Enable = function (self) getmetatable(self).__index.Enable(self) _G[self:GetName().."Text"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b) end
		if dependency:GetChecked() then
			control:Enable()
		else
			control:Disable()
		end
	else
		control.Disable = function (self) UIDropDownMenu_DisableDropDown(self) end
		control.Enable = function (self) UIDropDownMenu_EnableDropDown(self) end
	end

end

function EnergyWatchUI.SetupInverseDependentControl (dependency, control)
	if ( not dependency ) then
		return
	end
	
	assert(control)
	
	dependency.inverseDependentControls = dependency.inverseDependentControls or {}
	tinsert(dependency.inverseDependentControls, control)

	if ( control.type ~= CONTROLTYPE_DROPDOWN ) then
		control.Disable = function (self) getmetatable(self).__index.Disable(self) _G[self:GetName().."Text"]:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b) end
		control.Enable = function (self) getmetatable(self).__index.Enable(self) _G[self:GetName().."Text"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b) end
		if dependency:GetChecked() then
			control:Disable()
		else
			control:Enable()
		end
	else
		control.Disable = function (self) UIDropDownMenu_DisableDropDown(self) end
		control.Enable = function (self) UIDropDownMenu_EnableDropDown(self) end
	end

end